package routes_test

import (
	"bytes"
	"encoding/json"
	"gitea.deepak.science/deepak/gogmagog/models"
	"gitea.deepak.science/deepak/gogmagog/routes"
	"github.com/stretchr/testify/assert"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"
)

func TestPureJSONPutPlan(t *testing.T) {
	// set up
	assert := assert.New(t)
	p := &models.Plan{
		PlanID:          1,
		PlanDescription: "hn",
	}
	m := getEmptyModel()
	m.AddPlan(p, 3)
	router := routes.NewPlanRouter(m)
	data := []byte(`{
			"plan_id": 1,
			"plan_description": "testing"
		}`)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()
	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusOK, status)
	expected := `{
		"updated_plan": {
				"plan_description": "testing",
				"plan_id": 1,
				"user_id": 3
		},
		"id": 1
	}`
	assert.JSONEq(expected, rr.Body.String())
	contentType := rr.Header().Get("Content-Type")
	assert.Equal("application/json", contentType)
}

func TestExtraFieldPlanPutJSON(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	p := &models.Plan{
		PlanID:          1,
		PlanDescription: "hn",
	}
	m.AddPlan(p, 3)
	router := routes.NewPlanRouter(m)
	data := []byte(`{
			"plan_id": 1,
			"plan_description": "testing",
			"sabotage": "omg"
		}`)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()
	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusBadRequest, status)
	expected := `Bad Request`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestEmptyBodyPlanPut(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewPlanRouter(m)
	data := []byte(``)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()
	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusBadRequest, status)
	expected := `Bad Request`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestTwoBodyPlanPut(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewPlanRouter(m)
	data := []byte(`{
			"plan_id": 5
		}, {
				"plan_id": 6
			}`)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()
	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusBadRequest, status)
	expected := `Bad Request`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestBadPlanIDPut(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewPlanRouter(m)
	data := []byte(`{
			"plan_id": 5
		}, {
				"plan_id": 6
			}`)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/text", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()
	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusNotFound, status)
	expected := `Not Found`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestErrorUpdatePlan(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getErrorModel("Model always errors")

	router := routes.NewPlanRouter(m)
	p := &models.Plan{PlanID: 6}
	data, _ := json.Marshal(p)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)
	expected := `Internal Server Error`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestErrorOnRetrieveUpdatePlan(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getErrorOnGetModel("Model always errors")

	router := routes.NewPlanRouter(m)
	p := &models.Plan{PlanID: 6}
	data, _ := json.Marshal(p)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")
	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)
	expected := `Internal Server Error`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestErrorWriterUpdatePlan(t *testing.T) {
	// set up
	assert := assert.New(t)

	p := &models.Plan{PlanID: 6}
	m := getEmptyModel()
	m.AddPlan(p, 3)

	router := routes.NewPlanRouter(m)
	data, _ := json.Marshal(p)
	req, _ := http.NewRequestWithContext(sampleContext, "PUT", "/1", bytes.NewBuffer(data))
	req.Header.Set("Content-Type", "application/json")

	rr := NewBadWriter()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)

}
